package com.persagy.proxy.adm.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.dto.RequestData;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.basic.utils.QueryCriteriaHelper;
import com.persagy.dmp.common.constant.CommonConstant;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.digital.client.DigitalObjectFacade;
import com.persagy.dmp.digital.client.DigitalRelationCaclFacade;
import com.persagy.dmp.digital.entity.ObjectDigital;
import com.persagy.proxy.adm.constant.AdmObjectInfoConstant;
import com.persagy.proxy.adm.constant.AdmObjectType;
import com.persagy.proxy.adm.constant.AdmRelationTypeEnum;
import com.persagy.proxy.adm.constant.ObjTypeMapping;
import com.persagy.proxy.adm.dto.TwicePointDTO;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.AdmRelCalService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class AdmRelCalServiceImpl implements AdmRelCalService {

    /***
     * Description: 计算关系
     * @param context : 请求参数
     * @param admRelationTypeEnums : 关系类型集合
     * @param objectTypeStr : 参与计算的业务空间(为空表示所有业务空间类型)
     * @param relationTypeStr : 关系类型
     * return : void
     * @author : lijie
     * date :2021/9/11 20:35
     * Update By lijie 2021/9/11 20:35
     */
    @Override
    public void calcRelation(InstanceUrlParam context, List<AdmRelationTypeEnum> admRelationTypeEnums, String objectTypeStr,
                             String relationTypeStr) {
        for (AdmRelationTypeEnum admRelationTypeEnum : admRelationTypeEnums) {
            // 调用中台的计算逻辑
            DigitalRelationCaclFacade.calculatingObjRelationPrototype(context.getGroupCode(),context.getProjectId(),
                    context.getAppId(), CommonConstant.DEFAULT_ID, RequestData.builder()
                            .graphCode(admRelationTypeEnum.getGraphCode())
                            .relCode(admRelationTypeEnum.getRelCode())
                            .calBeforeRelFlag(true)
                            .build());
        }
    }
    /***
     * Description: 计算空间交通关系
     * @param context : 通用参数
     * @param buildingId : 建筑id
     * @param objectType : 对象类型
     * @param type : 类型
     * @param relationTypeEnum : 关系枚举类
     * @return : void
     * @author : lijie
     * @date :2021/9/30 14:49
     * Update By lijie 2021/9/30 14:49
     */
    @Override
    public AdmResponse calcRelationForArchForArchSpToSp(InstanceUrlParam context, String buildingId, String objectType,
                                                        String type, AdmRelationTypeEnum relationTypeEnum) {
        double distance = 100.00;
        if (StrUtil.isBlank(buildingId)){
            return AdmResponse.success(new ArrayList<>());
        }
        // 1.根据建筑id查询建筑下的所有楼层
        CommonResult<List<ObjectDigital>> commonResult = DigitalObjectFacade
                .queryObjectListSuperiorIdPrototype(context.getGroupCode(), context.getProjectId(),
                context.getAppId(), context.getUserId(), RequestData
                        .builder()
                        .buildingId(buildingId)
                        .objTypes(CollUtil.newHashSet(AdmObjectType.FLOOR.getIndex()))
                        .build());
        if (!"success".equals(commonResult.getResult())){
            return AdmResponse.failure(commonResult.getMessage());
        }
        Long count = commonResult.getCount();
        List<ObjectDigital> data = commonResult.getData();
        handleFloorRelationCalc(data,context);

        long skip = data.size();
        while (skip<count){
            // 有未处理完的楼层



        }


        return null;

    }
    /***
     * Description: 处理楼层下的关系计算(BDTP平台已经限制查询数据量)
     * @param data : 楼层数据
     * @param context: 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/9/30 16:46
     * Update By lijie 2021/9/30 16:46
     */
    private void handleFloorRelationCalc(List<ObjectDigital> data, InstanceUrlParam context) {
        if (CollUtil.isEmpty(data)){
            return;
        }
        // 2.循环所有楼层
        for (ObjectDigital floor : data) {
            if (!floor.getInfos().has(AdmObjectInfoConstant.MODEL_ID)
                    || StrUtil.isBlank(floor.getInfos().get(AdmObjectInfoConstant.MODEL_ID).asText())){
                continue;
            }
            // 2.1 根据楼层的modelId查询所有的门
            List<TwicePointDTO> pointDTOS =
                    queryAllDoorByFloorModelId(floor
                            .getInfos()
                            .get(AdmObjectInfoConstant.MODEL_ID)
                            .asText(),context);
            // 2.1 根据楼层id和classCode查询所有的空间


            // 2.3 遍历所有的门,得到门拥有的两组x,y坐标
            // 2.4 遍历坐标组,与空间进行对比,得到所有符合的空间id数组
            // 2.5 创建空间服务关系的关系数据


        }





    }
    /***
     * Description: 根据楼层模型id查询所有门数据(BDTP限制了查询条数,需要做分页查询)
     * @param floorModelId : 楼层模型id
     * @param context: 请求参数上下文
     * @return : java.util.List<com.persagy.proxy.adm.dto.TwicePointDTO>
     * @author : lijie
     * @date :2021/10/8 10:02
     * Update By lijie 2021/10/8 10:02
     */
    private List<TwicePointDTO> queryAllDoorByFloorModelId(String floorModelId, InstanceUrlParam context) {
        // 门的classCode:CFBEDR,obj_type为equipment
        QueryCriteria queryCriteria = new QueryCriteria();
        queryCriteria.getCriteria().put("classCode","CFBEDR")
                .put("objType",AdmObjectType.EQUIPMENT.getIndex())
                .put("modelId",floorModelId);
        List<ObjectNode> objectNodes = DigitalObjectFacade
                .query(context.getGroupCode(), context.getProjectId(), context.getAppId(),
                        context.getUserId(), queryCriteria);
        if (CollUtil.isEmpty(objectNodes)){
            return new ArrayList<>();
        }
        List<TwicePointDTO> result = new ArrayList<>();
        for (ObjectNode objectNode : objectNodes) {
            // 1.获得outline
            if (!objectNode.has("outline")
                    || !objectNode.get("outline").isArray()
                    || !(objectNode.get("outline").size()>2)){
                continue;
            }
            if (!objectNode.has("location")
                    || !objectNode.get("location").isObject()
                    || !objectNode.get("location").has("points")
                    || !objectNode.get("location").get("points").isArray()
                    || !(objectNode.get("location").get("points").size()>=1)){
                continue;
            }
            JsonNode point = objectNode.get("location").get("points").get(0);
            ArrayNode points = (ArrayNode) objectNode.get("outline").get(0);
            JsonNode firstPoint = points.get(0);
            JsonNode secondPoint = points.get(1);





        }


        return null;



    }
    /***
     * Description: 计算两点之间的距离(我不太明白这个算法是怎么算的)
     * @param x1 : 坐标点1的x值
     * @param y1 : 坐标点1的y值
     * @param x2 : 坐标点2的x值
     * @param y2 : 坐标点2的y值
     * @return : java.lang.Double  
     * @author : lijie
     * @date :2021/10/8 11:31
     * Update By lijie 2021/10/8 11:31
     */
    private Double getPoints(Double x1,Double y1,Double x2,Double y2){
        if (x1.equals(x2) && y1.equals(y2)){
            return null;
        }
        Double k=null;
        if (!x1.equals(x2)){
            k=(y2-y1)/(x2-x1);
        }
        if (null==k){
            return 0.00;
        }
        if (0.00 == k){
            return null;
        }
        return 1/k;
    }










}
